# -*- coding: utf-8 -*-

import datetime
import quopri
from base64 import b64encode

from django.conf import settings
from django.core.mail import send_mail
from django.http import HttpResponse, HttpResponseForbidden, Http404
from django.http import JsonResponse
from django.shortcuts import render, redirect
from django.template.loader import get_template
from django.urls import reverse
from django.utils import formats
from django.utils.html import escape
from django.utils.safestring import mark_safe

from club.models import Club
from ldap_inp import LdapEleve, LdapEleveAvecPhoto
from ldap_inp.ldapobject import LdapForeignKey
from lib.access_auth import access_restricted, access_inp
from portailuser.app_logger import AppLogger
from portailuser.models import Ecole, PortailUser
from . import search_tools
from .models import requete, Consultation_Annuaire_PortailUser, Consultation_Annuaire_IP

EXEMPLES_FILIERES = {
    "n7": ("EGE", "EN", "GEA", "Hydro", "AHY", "Info", "IR", "TR"),
    "ensat": ("Biotech", "Commune", "GECA", "IAA", "IAEGE", "PAFQP", "PVVA"),
    "a7": ("CH", "GC", "GI", "GPI", "IMAT", "LA", "MEP", "FIVA"),  # LA: oudated
    "enm": ("ENM"),
}
EXEMPLES_FILIERES["inp"] = EXEMPLES_FILIERES["n7"] + EXEMPLES_FILIERES["ensat"] + EXEMPLES_FILIERES["a7"]

CHAMPS_VCARD = [
    "ecole",
    "promo",
    "nickname",
    "homePhone",
    "mobile",
    "postalAddress",
    "skype",
    "personalWebsite",
    "mailForwardingAddress",
    "mailAnnexe",
    "birthdate",
    "filiere",
    "sn",
    "givenName",
]
# Seul ces champs sont autorisés pour les recherches multiples de vcard
CHAMPS_VCARD_RECHERCHE = ["ecole", "promo", "mailEcole", "filiere", "sn", "givenName"]

_logger_alerte = AppLogger("alerte_annuaire")


@access_inp
def recherche(request):
    texte_rech = request.GET.get("q", "").strip()

    resultats = []

    if texte_rech:
        resultats = requete(texte_rech, request.ecole)

        if isinstance(resultats, LdapEleve):
            return redirect("annuaire:fiche", uid=resultats.uid)

    c = {
        "q": texte_rech,
        "resultats": resultats,
        "filieres": EXEMPLES_FILIERES[request.ecole.ident],
    }

    return render(request, "annuaire/recherche.html", c)


def format_ddn(ddn):
    return formats.date_format(ddn, "SHORT_DATE_FORMAT")


def format_promo(promo):
    return "{} ({}A)".format(promo, PortailUser.extended_promo(promo))


def format_uri(protocole, donnee):
    return mark_safe('<a href="{}:{}">{}</a>'.format(protocole, escape(donnee.lower()), escape(donnee)))


def format_uris(protocole, donnees):
    return mark_safe("<br />\n".join([format_uri(protocole, i) for i in donnees if i]))


def format_urls(donnees, fiche=0):
    if fiche:
        # lien vers fiche annuaire
        url = mark_safe("<br />\n".join(['<a href="{}">{}</a>'.format(escape(i[0]), escape(i[1])) for i in donnees]))
    else:
        url = mark_safe("<br />\n".join(['<a href="{}">{}</a>'.format(escape(i), escape(i)) for i in donnees]))
    return url


def format_inscrit_ae(inscrit):
    if inscrit:
        retour = "\nOui\n"
    else:
        retour = "\nNon\n"
    return retour


def format_inscrit_pass_vie_etudiant(inscrit):
    if inscrit:
        retour = "\nOui\n"
    else:
        retour = "\nNon\n"
    return retour


def generate_fiche(request, uid, full=False, avertissement=0):
    try:
        eleve = LdapEleve.objects.get(uid=uid)
    except LdapEleve.DoesNotExist:
        raise Http404

    champs = {
        "ecole": {"nom": "École", "valeur": eleve.ecole.get().displayName},
        "filiere": {"nom": "Filière", "valeur": eleve.filiere.get().displayName},
        "promo": {"nom": "Promo", "valeur": format_promo(eleve.promo)},
        "birthdate": {"nom": "Date de naissance", "valeur": format_ddn(eleve.birthdate)},
        "nickname": {"nom": "Surnom", "valeur": eleve.nickname},
        "uid": {"nom": "Login INP-net", "valeur": eleve.uid},
        "loginTP": {"nom": "Login CRI", "valeur": eleve.loginTP},
        "inscritAE": {"nom": "Cotisant AE", "valeur": format_inscrit_ae(eleve.inscritAE)},
        "inscritPassVieEtudiant": {
            "nom": "Cotisant PassVie Étudiant",
            "valeur": format_inscrit_pass_vie_etudiant(eleve.inscritPassVieEtudiant),
        },
        "mailEcole": {"nom": "Mail école", "valeur": eleve.mailEcole},
        "skype": {"nom": "Skype", "valeur": eleve.skype},
        "personalWebsite": ({"nom": "Site perso", "valeur": eleve.personalWebsite},),
    }

    # TODO
    # if eleve.inscritPassVieEtudiant:
    #   champs["finPassVieEtudiant"] = {"nom": "Expiration du PassVie Étudiant", "valeur": eleve.finPassVieEtudiant}

    if full:
        champs.update(
            {
                "postalAddress": {"nom": "Adresse", "valeur": eleve.postalAddress},
                "homePhone": {"nom": "Numéro personnel", "valeur": eleve.homePhone},
                "mobile": {"nom": "Numéro portable", "valeur": eleve.mobile},
                "mailAnnexe": {"nom": "Mail annexe", "valeur": eleve.mailAnnexe},
                "mailLocal": {"nom": "Mails locaux", "valeur": eleve.mailLocal},
            }
        )

    # Affichage des groupes et des clubs
    groupes = ""
    for groupe in eleve.get_groupes():
        club = Club.objects.filter(ident=groupe.cn.rsplit("-", 1)[0])
        if club.exists():
            groupes += f"<a class='badge badge-pill badge-info' href='https:{club[0].get_absolute_url()}'>{escape(groupe.displayName)}</a><br>\n"
        else:
            groupes += f"<a class='badge badge-pill badge-secondary'>{escape(groupe.displayName)}</a><br>\n"

    if groupes:
        champs["groupes"] = {
            "nom": "Groupes",
            "valeur": mark_safe(f"<div style='font-size: 1.2rem; color: #FFF'>{groupes}</div>"),
        }

    c = {
        "champs": champs,
        "uid": eleve.uid,
        "nom": eleve.cn,
        "afficher_lien_modif": (request.user.is_authenticated and request.user.check_permission(request, "gerer_eleves") and not eleve.is_superuser()) or request.user.is_superuser,
        "afficher_lien_perms": request.user.is_authenticated
        and (request.user.is_staff or request.user.check_permission(request, "gerer_permissions")),
        "avertissement": avertissement,
        "full_fiche": full,
    }
    return render(request, "annuaire/fiche.html", c)


@access_inp
def fiche(request, uid):
    return generate_fiche(request, uid, False)


def acces_annuaire_limite(fun):
    def inner(request, *args, **kwargs):
        avertissement = 0
        limited_user = False
        if request.user.is_authenticated:
            consultation_annuaire = Consultation_Annuaire_PortailUser.objects.filter(
                user=request.user, date__gte=datetime.date.today()
            )
            if not consultation_annuaire:
                consultation_annuaire = Consultation_Annuaire_PortailUser(user=request.user, nb_acces=0)
            else:
                consultation_annuaire = consultation_annuaire[0]
            # pour les élèves diplomés ne disposant pas de la permission annuaire, ils sont bloqués à un seuil plus faible
            ldap_user = request.user.get_ldap_user()
            if request.user.check_permission(request, "consultation_annuaire"):
                # l'utilisateur a la permission pour ne pas être bloqué
                pass
            elif ldap_user.promo < datetime.date.today().year or (
                ldap_user.promo == datetime.date.today().year and datetime.date.today().month >= 11
            ):
                # les diplômés sont bloqués à SEUIL_BLOQUE_ADMIN à partir du mois de novembre de leur diplôme
                limited_user = True
        else:
            avertissement = 1
            consultation_annuaire = Consultation_Annuaire_IP.objects.filter(
                ip=request.META["REMOTE_ADDR"], date__gte=datetime.date.today()
            )
            if not consultation_annuaire:
                consultation_annuaire = Consultation_Annuaire_IP(ip=request.META["REMOTE_ADDR"], nb_acces=0)
            else:
                consultation_annuaire = consultation_annuaire[0]
        consultation_annuaire.nb_acces += 1
        consultation_annuaire.save()

        if limited_user and consultation_annuaire.nb_acces >= settings.SEUIL_BLOQUE_ADMIN:
            avertissement = 4
            if consultation_annuaire.nb_acces == settings.SEUIL_BLOQUE_ADMIN:
                message = "Yop,\n\nL'utilisateur {} a fait plus de {} recherches sur l'annuaire aujourd'hui. Sa dernière IP est {}\n" "Son accès à l'annuaire est maintenant limité.\n\nJe vous laisse, j'ai des requêtes qui m'attendent.\n\nLe portail".format(
                    request.user,
                    settings.SEUIL_BLOQUE_ADMIN,
                    request.META["REMOTE_ADDR"],
                )
                send_mail(
                    "Alerte Annuaire",
                    message,
                    settings.DEFAULT_FROM_EMAIL,
                    ["portail@bde.inp-toulouse.fr"],
                    fail_silently=False,
                )
                _logger_alerte.log(
                    request.user,
                    "A demandé plus de {} fiches".format(settings.SEUIL_BLOQUE_ADMIN),
                )

        elif limited_user and consultation_annuaire.nb_acces >= (settings.SEUIL_BLOQUE_ADMIN // 2):
            avertissement = 3

        elif consultation_annuaire.nb_acces >= settings.SEUIL_ALERTE_ADMIN:
            avertissement = 4
            if consultation_annuaire.nb_acces == settings.SEUIL_ALERTE_ADMIN:
                message = "Yop,\n\nL'utilisateur {} a fait plus de {} recherches sur l'annuaire aujourd'hui. Sa dernière IP est {}\n" "\nJe vous laisse, j'ai des requêtes qui m'attendent.\n\nLe portail".format(
                    request.user,
                    settings.SEUIL_ALERTE_ADMIN,
                    request.META["REMOTE_ADDR"],
                )
                send_mail(
                    "Alerte Annuaire",
                    message,
                    settings.DEFAULT_FROM_EMAIL,
                    ["portail@bde.inp-toulouse.fr"],
                    fail_silently=False,
                )
                if request.user.is_authenticated:
                    _logger_alerte.log(
                        request.user,
                        "A demandé plus de {} fiches".format(settings.SEUIL_ALERTE_ADMIN),
                    )
        elif consultation_annuaire.nb_acces >= settings.SEUIL_AVERTISSEMENT:
            avertissement = 1

        return fun(request, avertissement, *args, **kwargs)

    return inner


@access_inp
@acces_annuaire_limite
# 'avertissement' est un entier passé par le décorateur 'acces_annuaire_limite'
def fiche_full(request, avertissement, uid):
    if avertissement >= 4:
        return generate_fiche(request, uid, False, avertissement)
    return generate_fiche(request, uid, True, avertissement)


@access_inp
def photo(request, uid):
    static_url = settings.STATIC_URL
    if request.is_secure:
        static_url = static_url.replace("http:", "https:")

    try:
        eleve = LdapEleveAvecPhoto.objects.get(uid=uid)
    except LdapEleveAvecPhoto.DoesNotExist:
        raise Http404

    photo = eleve.jpegPhoto

    if photo:
        return HttpResponse(photo.rawdata, content_type="image/jpeg")
    elif eleve.genre == 1:
        return redirect(static_url + "annuaire/anonyme_m.png")
    else:
        return redirect(static_url + "annuaire/anonyme_f.png")


def valeur_vcard(valeur):
    if isinstance(valeur, list):
        # TODO Autoriser plusieurs valeurs
        try:
            valeur = valeur[0]
        except IndexError:
            return None

    if valeur is None:
        return None

    if isinstance(valeur, LdapForeignKey):
        valeur = valeur.get().displayName

    return (
        mark_safe(quopri.encodestring(str.encode(str(valeur), "utf8")).decode("utf8"))
        .replace("\n", "=0D=0A")
        .replace("\r", "")
    )


@access_inp
@acces_annuaire_limite
# 'avertissement' est un entier passé par le décorateur 'acces_annuaire_limite'
def vcard_uid(request, avertissement, uid):
    if avertissement >= 4:
        return HttpResponseForbidden(
            "Vous avez un usage excessif de l'annuaire, merci de vous justifier "
            "auprès d'INP-net, sous risque de suppression de votre compte "
        )

    try:
        eleve = LdapEleveAvecPhoto.objects.get(uid=uid)
    except LdapEleveAvecPhoto.DoesNotExist:
        raise Http404

    photo = b64encode(eleve.jpegPhoto.rawdata).decode("utf8") if eleve.jpegPhoto else None

    donnees = {}
    for champ in CHAMPS_VCARD:
        donnees[champ] = valeur_vcard(getattr(eleve, champ))

    c = {"u": donnees, "photo": photo, "date": datetime.datetime.now()}

    t = get_template("annuaire/vcard.txt")

    reponse = HttpResponse(t.render(context=c).encode("ascii", "ignore"), content_type="text/vcard")
    reponse["Content-Disposition"] = "attachment;filename={}.vcf".format(uid)
    return reponse


@access_inp
@acces_annuaire_limite
# 'avertissement' est un entier passé par le décorateur 'acces_annuaire_limite'
def vcard_rech(request, avertissement):
    print(avertissement)
    if avertissement >= 4:
        return HttpResponseForbidden(
            "Vous avez un usage excessif de l'annuaire, merci de vous justifier "
            "auprès d'INP-net, sous risque de suppression de votre compte "
        )

    texte_rech = request.GET.get("q", "").strip()

    if not texte_rech:
        raise Http404

    resultats = requete(texte_rech, request.ecole if request.ecole != Ecole.inp else None, True)

    if not resultats:
        raise Http404

    if isinstance(resultats, LdapEleveAvecPhoto) or len(resultats) < 2:
        return redirect("annuaire:vcard_uid", uid=resultats.uid)

    t = get_template("annuaire/vcard_recherche.txt")
    reponse = HttpResponse(content_type="text/vcard")
    reponse["Content-Disposition"] = "attachment;filename={}.vcf".format(texte_rech.replace(" ", "_"))

    for eleve in resultats:
        # On ne renvoie plus les photos dans les vcard
        # photo = b64encode(eleve.jpegPhoto.rawdata) if eleve.jpegPhoto else None

        donnees = {}
        for champ in CHAMPS_VCARD_RECHERCHE:
            donnees[champ] = valeur_vcard(getattr(eleve, champ))

        c = {
            "u": donnees,
            # 'photo': photo,
            "date": datetime.datetime.now(),
        }

        reponse.write(t.render(context=c))

    return reponse


@access_restricted
def adm_recherche(request, titre, redir, **kwargs):
    texte_rech = request.GET.get("q", "").strip()

    resultats = []

    if texte_rech:
        resultats = requete(texte_rech, request.ecole if request.ecole != Ecole.inp else None)

        if isinstance(resultats, LdapEleve):
            return redirect(redir, uid=resultats.uid, **kwargs)

        for eleve in resultats:
            kwargs["uid"] = eleve.uid
            object.__setattr__(eleve, "url", reverse(redir, kwargs=kwargs))

    c = {
        "titre": titre,
        "q": texte_rech,
        "resultats": resultats,
    }

    return render(request, "annuaire/adm_recherche.html", c)


@access_inp
def mini_recherche(request):
    if request.method == "GET":
        try:
            recherche = request.GET["recherche"]
            champs = ("snSearch", "givenNameSearch", "nickname")
            for combi in search_tools.combispace(recherche.split()):
                filtre = search_tools.gen_filterldap(combi, champs)
                if filtre is not None:
                    resultats = LdapEleve.objects.filter(filtre)
                    if len(resultats) != 0:
                        break
            reponses = []
            for eleve in resultats[:10]:
                reponses.append(
                    {
                        "prenom": eleve.prenom,
                        "nom": eleve.nom,
                        "uid": eleve.uid,
                        "photo_url": reverse("annuaire:photo", args=(eleve.uid,)),
                    }
                )
            return JsonResponse(reponses, safe=False)
        except Exception:
            return JsonResponse({"bad": "error"})


@access_restricted
def informations(request, uid):
    try:
        eleve = LdapEleve.objects.get(uid=uid)
    except LdapEleve.DoesNotExist:
        raise Http404

    donnees = "<a href='{}'>{} {} • {} {}</a>".format(
        reverse("annuaire:fiche", args=[str(uid)]),
        eleve.prenom,
        eleve.nom,
        eleve.filiere.get().shortName,
        eleve.promo,
    )

    return HttpResponse(donnees, content_type="text/plain;charset=utf8")


def opensearch(request):
    c = {
        "hote": ("https://" if request.is_secure() else "http://") + request.get_host(),
    }
    return render(
        request,
        "annuaire/opensearch.xml",
        c,
        content_type="application/opensearchdescription+xml",
    )


@access_restricted
def arbre_genealogique_api(request, uid):
    return JsonResponse(arbre_genealogique(racine(uid), {}))


def fillots(uid):
    return [e.uid for e in LdapEleve.objects.filter(uidParrain=uid)]


def racine(uid):
    """Obtenir l'uid de la racine de l'arbre généalogique d'un élève."""
    eleve = LdapEleve.objects.get(uid=uid)
    while eleve.uidParrain:
        eleve = LdapEleve.objects.get(uid=eleve.uidParrain)
    return eleve.uid


def arbre_genealogique(uid, arbre):
    """Obtenir un arbre généalogique à partir d'un uid correspondant à la racine de l'arbre."""
    eleve = LdapEleve.objects.get(uid=uid)
    arbre[eleve.uid] = (eleve.displayName, eleve.uidParrain)
    for f in fillots(uid):
        arbre_genealogique(f, arbre)
    return arbre
